#include "beamtreescene.h"

BeamTreeScene::BeamTreeScene(QObject *parent) :
	QGraphicsScene(parent),
	timerRunning(false)
{
	pen.setWidthF(0);
}

void BeamTreeScene::render(node *dir) {
	dir->setPen(pen);
	addItem(dir);
	QVector<node*> children = dir->getChildren();
	for (int i=0; i < children.size(); i++) {
		node *n = children[i];
		QRectF rect = n->calculateRect();
		n->setZValue(n->getDepth());
		n->addParentOffset(rect);
		
		QLinearGradient lingrad;
		
		if (n->isVertical())
		{
			lingrad.setStart(n->rect().topLeft().x(), n->rect().topLeft().y() + n->rect().height()/2);
			lingrad.setFinalStop(n->rect().bottomRight().x(), n->rect().topLeft().y() + n->rect().height()/2);
		}
		else
		{
			lingrad.setStart(n->rect().topLeft().x() + n->rect().width()/2, n->rect().topLeft().y());
			lingrad.setFinalStop(n->rect().topLeft().x() + n->rect().width()/2, n->rect().bottomRight().y());

		}
		
		if (n->isDir()){
			lingrad.setColorAt(0, QColor(50,50,50,150));
			lingrad.setColorAt(0.5, QColor(150,150,150,150));
			lingrad.setColorAt(1, QColor(50,50,50,150));
			n->setBrush(QBrush(lingrad));
		}
		else{
			lingrad.setColorAt(0, QColor(50,50,255,150));
			lingrad.setColorAt(0.5, QColor(100,100,255,150));
			lingrad.setColorAt(1,  QColor(50,50,255, 150));
			n->setBrush(QBrush(lingrad));
		}
		n->setPen(QPen());
		addItem(n);
		render(n);
	}
}

void BeamTreeScene::draw() {
	draw(1.0);
}

void BeamTreeScene::draw(float scale) {
	QList<QGraphicsItem *> list = items();
	QListIterator<QGraphicsItem *> it(list);
	while(it.hasNext()) {
		
		node *n = (node *) it.next();
		n->resetTransform();
		QPointF origin = n->rect().topLeft();
		if (n->isDir()) {
			if (n->isVertical()) {
				n->translate(n->rect().center().x(),n->rect().center().y() );
				n->scale(1,scale);
				n->translate((-1)*n->rect().center().x(),(-1)*n->rect().center().y() );
			} else {
				n->translate(n->rect().center().x(),n->rect().center().y() );
				n->scale(scale,1);
				n->translate((-1)*n->rect().center().x(),(-1)*n->rect().center().y() );
			}
		}
	}
}

void BeamTreeScene::setScale(int scale) {
	float fScale = ((float)scale) / 100;
	draw(fScale);
}

void BeamTreeScene::reset() {
	clear();
}

void BeamTreeScene::hilight(node *n) {
	if (timerRunning) return;
	oldBrush = n->brush();
	oldNode = n;
	textItem = new QGraphicsSimpleTextItem(n->getPath(), n);
	textItem->setPos(n->rect().topLeft());
	textItem->scale(2,2);
	n->setBrush(QBrush(QColor(255,0,0,200)));
	n->update(n->rect());
	timerRunning = true;
	QTimer::singleShot(500, this, SLOT(resetColor()));
}

void BeamTreeScene::resetColor() {
	textItem->setVisible(false);
	oldNode->setBrush(oldBrush);
	oldNode->update(oldNode->rect());
	timerRunning = false;
}
