#include "mainwindow.h"
#include <iostream>

using namespace std;

MainWindow::MainWindow() {
	createActions();
	createToolBar();
	createDock();
	createCentralWidget();

	scale = 1.0;

	setUnifiedTitleAndToolBarOnMac(true);

	connect(openAction, SIGNAL(triggered()), this, SLOT(chooseFolder()));
	connect(zoomInAction, SIGNAL(triggered()), this, SLOT(zoomIn()));
	connect(zoomOutAction, SIGNAL(triggered()), this, SLOT(zoomOut()));
	connect(slider, SIGNAL(valueChanged(int)), scene, SLOT(setScale(int)));
	connect( treeWidget, SIGNAL(itemClicked(QTreeWidgetItem*,int)),
			this, SLOT(setRoot(QTreeWidgetItem*,int)) );
}

void MainWindow::createActions() {
	// open
	openAction = new QAction(QIcon(":/images/open.png"), tr("&Open Folder"), this);
	openAction->setShortcut(tr("Ctrl+O"));
	openAction->setStatusTip(tr("Select a folder to visualize..."));

	// zoom in
	zoomInAction = new QAction(QIcon(":/images/zoomin.png"), tr("&Zoom In"), this);
	zoomInAction->setShortcut(tr("Ctrl++"));
	zoomInAction->setStatusTip(tr("Zoom out..."));

	// zoom out
	zoomOutAction = new QAction(QIcon(":/images/zoomout.png"), tr("&Zoom Out"), this);
	zoomOutAction->setShortcut(tr("Ctrl+-"));
	zoomOutAction->setStatusTip(tr("Zoom in..."));
}

void MainWindow::createToolBar() {
	toolBar = addToolBar("Main");
	toolBar->addAction(openAction);
	toolBar->addAction(zoomInAction);
	toolBar->addAction(zoomOutAction);

	slider = new QSlider(Qt::Horizontal, this);
	slider->setRange(0, 100);
	slider->setValue(100);
	slider->setMaximumWidth(150);
	toolBar->addWidget(slider);
}

void MainWindow::createDock() {
	treeWidget = new QTreeWidget();
	treeWidget->setColumnCount(2);
	treeWidget->setHeaderLabels( QStringList() << "dir" << "size" );

	node::setTreeWidget(treeWidget);
	
	dockWidget = new QDockWidget("Directory Tree", this);
	dockWidget->setWidget(treeWidget);
	dockWidget->setTitleBarWidget(new QWidget);

	addDockWidget(Qt::LeftDockWidgetArea, dockWidget);
}

void MainWindow::createCentralWidget() {
	scene = new BeamTreeScene;
	view = new QGraphicsView(scene);
	scene->setView(view);
	setCentralWidget(view);
}

void MainWindow::chooseFolder() {
	QString dir = QFileDialog::getExistingDirectory(
			this,
			"Choose directory...",
			"",
			QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks
			);

	qDebug() << "dir:" << dir;

	node *n = new node(dir);
	n->setRect( 0, 0, 500, 500 );
	
	n->traverse();
	treeWidget->clear();
	treeWidget->addTopLevelItem(fillTree(n));
	scene->reset();
	scene->render(n);
}


TreeItem* MainWindow::fillTree(node *root, TreeItem *rootItem){
	
	if (rootItem == 0)
	{
		QStringList data;
		data << root->getPath() << QString::number(root->getTotalSize());
		TreeItem *rootItem = new TreeItem(data);
		rootItem->setNode(root);
		fillTree(root, rootItem);
		root->setTreeItem(rootItem);
		return rootItem;
	}
	else
	{
		QVector<node*> children = root->getChildren();
		for (int i=0; i < children.size(); i++) {
				node *n = children[i];
				QStringList data;
				data << n->getPath() << QString::number(n->getTotalSize());
				TreeItem *child = new TreeItem(rootItem, data);
				child->setNode(n);
				rootItem->addChild(child);
				rootItem->setExpanded(true);
				n->setTreeItem(child);
				if (n->isDir()) {
					fillTree(n, child);
				}
		}
		return 0;
	}
}

void MainWindow::zoomIn() {
	scale += 0.1;
	view->resetTransform();
	view->scale(scale, scale);
}

void MainWindow::zoomOut() {
	scale -= 0.1;
	if (scale < 0.5) scale = 0.5;
	view->resetTransform();
	view->scale(scale, scale);
}

void MainWindow::setRoot(QTreeWidgetItem *item, int column) {
	TreeItem *treeItem = (TreeItem *) item;
	node *n = treeItem->getNode();
	scene->hilight(n);
}
