#include "node.h"

float node::scaleFactor = 1.0;
QTreeWidget * node::treeWidget = NULL;

node::node(QString path, QObject *parent) :
	QObject(parent),
	depth(0),
	fileSize(0),
	totalSize(0),
	offset(0)
{
	this->path = path;
}

void node::addChild(node *childNode) {
	children.append(childNode);

	if (!childNode->bDir) {
		childNode->setParentItem(this);
	}
}

void node::addToTotalSize(qint64 size) {
	totalSize += size;
}

void node::traverse() {
	QDir dir(path);
	QDirIterator it(dir);
	traverse(it);
}

qint64 node::traverse(QDirIterator& it) {
	while (it.hasNext()) {
		it.next();

		if (it.fileName() == "." || it.fileName() == "..")
			continue;

		QFileInfo fileInfo = it.fileInfo();
		QString indent;
		for (int i=0; i<depth; i++) {
			indent += "  ";
		}
		node *childNode = new node(it.fileName(), this);
		childNode->setSize(fileInfo.size());
		childNode->addToTotalSize(fileInfo.size());
		childNode->bDir = fileInfo.isDir();
		childNode->depth = depth+1;
		
		addChild(childNode);

		addToTotalSize( childNode->getSize() );
		
		if (fileInfo.isDir()) {
			QDir dir(it.filePath());
			QDirIterator subIt(dir);

			addToTotalSize(childNode->traverse(subIt));
		}
	}
	return totalSize;
}

float node::getRelativeSize() {
	//qDebug() << totalSize;
	//qDebug() << parent()->totalSize;
	float div = totalSize / parent()->totalSize;
	//qDebug() << div;
	return totalSize / parent()->totalSize;

}

bool node::isVertical() {
	return (depth % 2 == 1);
}

QRectF node::calculateRect() {
	node *p = parent();
	if (p == NULL)
		return rect();

	float x, y, w, h;
	
	if (isVertical()) {
		// add offset to y-axis
		float newY = p->rect().y() + p->getOffset();
		float newH = p->rect().height() * getRelativeSize();

		x = p->rect().x();
		y = newY;
		w = p->rect().width();
		h = newH;
	} else {
		// add offset to x-axis
		float newX = p->rect().x() + p->getOffset();
		//qDebug() << newX << " " << p->getOffset() << " " << p->rect().x();
		float newW = p->rect().width() * getRelativeSize();

		x = newX;
		y = p->rect().y();
		w = newW;
		h = p->rect().height();
	}

	setRect(x, y, w, h);
	//qDebug() << x << " " << y << " " << w << " " << h;
	return rect();
}

void node::addParentOffset(float offset) {
	node *p = parent();
	if (p == NULL) return;
	p->offset += offset;
}

void node::addParentOffset(QRectF rect) {
	float offset;
	if (isVertical())
		offset = rect.height();
	else offset = rect.width();
	addParentOffset(offset);
}

node* node::parent() {
	return (node *) QObject::parent();
}

void node::mousePressEvent(QGraphicsSceneMouseEvent *event) {
	qDebug();
	if (parent() == NULL)
		qDebug() << "clicked:" << path << "isVertical:" << isVertical();
	else
		qDebug() << "clicked:" << path << "isVertical:" << isVertical() << "parent:" << parent()->path;
	qDebug() << "  rect:       " << rect();
	qDebug() << "  mapped rect:" << mapToScene(rect()).boundingRect();
	if (parent() == NULL) return;
	qDebug() << "  parent rect:" << parent()->rect();
	qDebug() << "  par. rect m:" << (parent()->mapToScene(rect())).boundingRect();
	qDebug() << "  pos:" << event->pos() << " scenePos:" << event->scenePos();
	qDebug() << "  buttonDownPos:" << event->buttonDownPos(Qt::LeftButton) << " screenPos:" << event->screenPos();

	if (treeWidget != NULL && treeItem != NULL) {
		treeWidget->setCurrentItem(treeItem);
	}
}
