#ifndef NODE_H
#define NODE_H

#include "includes.h"

//!  Repräsentiert einen Knoten im Verzeichnisbaum.
/*!
  Die Klasse node repräsentiert ein Verzeichnis bzw. eine Datei im Verzeichnisbaum.

  Ein node-Objekt speichert jeweils Pfad (#path), Typ (#bDir) Größe (#fileSize), sowie einen Zeiger auf den Elternknoten (parent()). Verzeichnisse speichern zudem eine Liste der untergeordneten Dateien und Verzeichnisse (#children), sowie deren Gesamtgröße (#totalSize).
*/

class node : public QObject, public QGraphicsRectItem
{
	Q_OBJECT
	
	public:
		node(QString path, QObject *parent=0);

		void addToTotalSize(qint64 size);
		void traverse();
		bool isDir() { return bDir; }
		QRectF calculateRect();
		void addParentOffset(float offset);
		void addParentOffset(QRectF rect);
		//! Liste von untergeordneten Knoten.
		/*!
		    \see getChildren()
		 */
		QVector<node*> children;
		node* parent();
		bool isVertical();

		qint64 getTotalSize() { return totalSize; }
		QString getPath() { return path; }
		QVector<node*> getChildren() { return children; }
		int getDepth() { return depth; }
		float getScaleFactor() { return scaleFactor; }
		void setTreeItem(QTreeWidgetItem *item) { treeItem = item; }
		
		static void setTreeWidget(QTreeWidget *tw) { treeWidget = tw; }
		
	protected:
		static float scaleFactor;
		static QTreeWidget *treeWidget;
		
		//! Pfad des aktuellen Knotens (relativ zum Elternknoten).
		QString path;
		//! Tiefe aktuellen Knotens.
		int depth;
		//! Größe des aktuellen Knotens.
		/*!
		    \see getSize(), #totalSize
		 */
		qint64 fileSize;
		//! Größe des aktuellen Unterbaumes
		/*!
		    \see getTotalSize(), #fileSize
		 */
		float totalSize;
		float offset;
		bool bDir;
		QTreeWidgetItem *treeItem;

		qint64 traverse(QDirIterator& it);
		void addChild(node *childNode);
		float getRelativeSize();
		void mousePressEvent(QGraphicsSceneMouseEvent *event);

		void setSize(qint64 size) { fileSize = size; }
		qint64 getSize() { return fileSize; }
		float getOffset() { return offset; }
};

#endif
